@@ -38,7 +38,6 @@ import org.json.JSONException;
 import org.json.JSONObject;
 import org.mozilla.gecko.annotation.RobocopTarget;
 
-import static org.mozilla.gecko.util.ContentUriUtils.getOriginalFilePathFromUri;
 import static org.mozilla.gecko.util.ContentUriUtils.getTempFilePathFromContentUri;
 
 public class FileUtils {
@@ -300,14 +299,10 @@ public class FileUtils {
     }
 
     public static String resolveContentUri(final Context context, final Uri uri) {
-        String path = getOriginalFilePathFromUri(context, uri);
-        if (TextUtils.isEmpty(path)) {
-            // We cannot always successfully guess the original path of the file behind the
-            // content:// URI, so we need a fallback. This will break local subresources and
-            // relative links, but unfortunately there's nothing else we can do
-            // (see https://issuetracker.google.com/issues/77406791).
-            path = getTempFilePathFromContentUri(context, uri);
-        }
+        // This will break local subresources and relative links, but
+        // unfortunately there's nothing else we can do (see
+        // https://issuetracker.google.com/issues/77406791).
+        final String path = getTempFilePathFromContentUri(context, uri);
         return !TextUtils.isEmpty(path) ? String.format(FILE_ABSOLUTE_URI, path) : path;
     }
 
@@ -324,7 +319,7 @@ public class FileUtils {
             e.printStackTrace();
         }
 
-        return fileName;
+        return canonicalizeFilename(fileName);
     }
 
     public static void copy(final Context context, final Uri srcUri, final File dstFile) {
@@ -416,4 +411,12 @@ public class FileUtils {
             return absPath.contains("/sdcard0") || absPath.contains("/storage/emulated");
         }
     }
+
+    private static @Nullable String canonicalizeFilename(@Nullable final String originalFilename) {
+        if (TextUtils.isEmpty(originalFilename)) {
+            return null;
+        } else {
+            return new File(originalFilename).getName();
+        }
+    }
 }
