@@ -3,17 +3,19 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* eslint-disable mozilla/use-chromeutils-import */
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.defineModuleGetter(
+  this,
+  "RuntimePermissions",
+  "resource://gre/modules/RuntimePermissions.jsm"
+);
 
-XPCOMUtils.defineLazyModuleGetter(this, "RuntimePermissions", "resource://gre/modules/RuntimePermissions.jsm");
-
-add_task(function* test_snackbar_api() {
+add_task(async function test_snackbar_api() {
   RuntimePermissions.waitForPermissions([
     RuntimePermissions.CAMERA,
     RuntimePermissions.RECORD_AUDIO,
-    RuntimePermissions.WRITE_EXTERNAL_STORAGE
+    RuntimePermissions.WRITE_EXTERNAL_STORAGE,
   ]);
 });
 
